// Copyright 2022 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
// 
//     https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

//
//  ViewController.swift
//  ImageClassifierStep2
//
//  Created by Laurence Moroney on 3/31/21.
//

import UIKit
import MLKitVision
import MLKit
import MLKitImageLabelingCommon
import MLKitImageLabelingCustom

class ViewController: UIViewController {

    @IBAction func doClassification(_ sender: Any) {
        getLabels(with: imageView.image!)
    }
    @IBOutlet weak var imageView: UIImageView!
    @IBOutlet weak var lblOutput: UILabel!
    override func viewDidLoad() {
        super.viewDidLoad()
        // Do any additional setup after loading the view.
        imageView.image = UIImage(named:"flower2.jpg")
    }
    
    // This is called when the user presses the button
    func getLabels(with image: UIImage){
        // Get the image from the UI Image element and set its orientation
        let visionImage = VisionImage(image: image)
        visionImage.orientation = image.imageOrientation
        // Add this code to use a custom model
        let localModelFilePath = Bundle.main.path(forResource: "model", ofType: "tflite")
        let localModel = LocalModel(path: localModelFilePath!)
        
        // Create Image Labeler options, and set the threshold to 0.4
        // so we will ignore all classes with a probability of 0.4 or less
        let options = CustomImageLabelerOptions(localModel: localModel)
        options.confidenceThreshold = 0.4
        
        // Initialize the labeler with these options
        let labeler = ImageLabeler.imageLabeler(options: options)
        
        // And then process the image, with the callback going to self.processresult
        labeler.process(visionImage) { labels, error in
            self.processResult(from: labels, error: error)
     }
    }
    
    // This gets called by the labeler's callback
    func processResult(from labels: [ImageLabel]?, error: Error?){
        // String to hold the labels
        var labeltexts = ""
        // Check that we have valid labels first
        guard let labels = labels else{
            return
        }
        // ...and if we do we can iterate through the set to get the description and confidence
        for label in labels{
            let labelText = label.text + " : " + label.confidence.description + "\n"
            labeltexts += labelText
        }
        // And when we're done we can update the UI with the list of labels
        lblOutput.text = labeltexts
    }

}

